@@ -1,7 +1,7 @@
 /*
  * guile-bin.c - binary data exchange layer for Guile servers
  *
- * Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile-bin.c,v 1.14 2001/11/28 18:04:41 ela Exp $
+ * $Id: guile-bin.c,v 1.19 2002/02/15 12:16:07 ela Exp $
  *
  */
 
@@ -66,8 +66,9 @@ static long guile_bin_tag = 0;
   if (!CHECK_BIN_SMOB (binary))                                    \
     scm_wrong_type_arg_msg (FUNC_NAME, arg, binary, "svz-binary"); \
   var = GET_BIN_SMOB (binary)
-#define MAKE_BIN_SMOB() \
-  ((guile_bin_t *) scm_must_malloc (sizeof (guile_bin_t), "svz-binary"))
+#define MAKE_BIN_SMOB()                                    \
+  ((guile_bin_t *) ((void *)                               \
+    scm_must_malloc (sizeof (guile_bin_t), "svz-binary")))
 
 /* Smob test function: Returns @code{#t} if the given cell @var{binary} is 
    an instance of the binary smob type. */
@@ -253,7 +254,7 @@ guile_bin_set_x (SCM binary, SCM index, SCM value)
   /* Check the range of the index argument. */
   idx = SCM_NUM2INT (SCM_ARG2, index);
   if (idx < 0 || idx >= bin->size)
-    scm_out_of_range_pos (FUNC_NAME, index, SCM_ARG2);
+    SCM_OUT_OF_RANGE (SCM_ARG2, index);
 
   bin->data[idx] = SCM_CHAR (value);
   return SCM_UNSPECIFIED;
@@ -275,7 +276,7 @@ guile_bin_ref (SCM binary, SCM index)
   /* Check the range of the index argument. */
   idx = SCM_NUM2INT (SCM_ARG2, index);
   if (idx < 0 || idx >= bin->size)
-    scm_out_of_range_pos (FUNC_NAME, index, SCM_ARG2);
+    SCM_OUT_OF_RANGE (SCM_ARG2, index);
 
   return SCM_MAKE_CHAR (bin->data[idx]);
 }
@@ -360,9 +361,9 @@ guile_bin_subset (SCM binary, SCM start, SCM end)
 
   /* Check the ranges of both indices. */
   if (from < 0 || from >= bin->size || from >= end)
-    scm_out_of_range_pos (FUNC_NAME, from, SCM_ARG2);    
+    SCM_OUT_OF_RANGE (SCM_ARG2, start);
   if (to < 0 || to >= bin->size || to <= from)
-    scm_out_of_range_pos (FUNC_NAME, to, SCM_ARG3);    
+    SCM_OUT_OF_RANGE (SCM_ARG3, end);
 
   ret = MAKE_BIN_SMOB ();
   ret->size = to - from + 1;
@@ -422,7 +423,7 @@ guile_list_to_bin (SCM list)
     {
       if (!SCM_EXACTP (SCM_CAR (list)))
 	scm_out_of_range (FUNC_NAME, SCM_CAR (list));
-      value = SCM_NUM2INT (SCM_ARG1, SCM_CAR (list));
+      value = SCM_NUM2INT (SCM_ARGN, SCM_CAR (list));
       if (value < 0 || value > 255)
 	scm_out_of_range (FUNC_NAME, SCM_CAR (list));
       *p++ = value;
@@ -468,16 +469,124 @@ guile_bin_to_data (SCM binary, int *size)
   return bin->data;
 }
 
+/* The following macro expands to a function definition which accesses a
+   binary smob's data for reading depending on the given @var{ctype}. */
+#define MAKE_BIN_REF(ctype)                                                  \
+static SCM GUILE_CONCAT3 (guile_bin_,ctype,_ref) (SCM binary, SCM index) {   \
+  guile_bin_t *bin; int idx;                                                 \
+  CHECK_BIN_SMOB_ARG (binary, SCM_ARG1, bin);                                \
+  SCM_ASSERT_TYPE (SCM_EXACTP (index), index, SCM_ARG2, FUNC_NAME, "exact"); \
+  idx = SCM_NUM2INT (SCM_ARG2, index);                                       \
+  if (idx < 0 || idx >= (int) (bin->size / sizeof (ctype)))                  \
+    SCM_OUT_OF_RANGE (SCM_ARG2, index);                                      \
+  return scm_long2num ((long) ((ctype *) bin->data)[idx]);                   \
+}
+
+/* Checks whether the scheme value @var{value} can be stored within a
+   @var{ctype}. The macro stores valid values in @var{val} and throws an
+   exception if it is out of range. */
+#define CTYPE_CHECK_RANGE(ctype, value, pos, val) do {                 \
+    if (SCM_POSITIVEP (value)) {                                       \
+      unsigned long uval = SCM_NUM2ULONG (pos, value);                 \
+      unsigned ctype cval = (unsigned ctype) uval;                     \
+      if (uval != (unsigned long) cval) SCM_OUT_OF_RANGE (pos, value); \
+      val = (unsigned long) uval;                                      \
+    } else {                                                           \
+      signed long ival = SCM_NUM2LONG (pos, value);                    \
+      signed ctype cval = (signed ctype) ival;                         \
+      if (ival != (signed long) cval) SCM_OUT_OF_RANGE (pos, value);   \
+      val = (unsigned long) ival;                                      \
+    }                                                                  \
+  } while (0)
+
+/* The following macro expands to a function definition which accesses a
+   binary smob's data for writing depending on the given @var{ctype}. */
+#define MAKE_BIN_SET(ctype)                                                  \
+static SCM GUILE_CONCAT3 (guile_bin_,ctype,_set) (SCM binary, SCM index,     \
+						  SCM value) {               \
+  guile_bin_t *bin; int idx; unsigned long val; SCM old; ctype *data;        \
+  CHECK_BIN_SMOB_ARG (binary, SCM_ARG1, bin);                                \
+  SCM_ASSERT_TYPE (SCM_EXACTP (index), index, SCM_ARG2, FUNC_NAME, "exact"); \
+  idx = SCM_NUM2INT (SCM_ARG2, index);                                       \
+  if (idx < 0 || idx >= (int) (bin->size / sizeof (ctype)))                  \
+    SCM_OUT_OF_RANGE (SCM_ARG2, index);                                      \
+  SCM_ASSERT_TYPE (SCM_EXACTP (value), value, SCM_ARG3, FUNC_NAME, "exact"); \
+  CTYPE_CHECK_RANGE (ctype, value, SCM_ARG3, val);                           \
+  data = (ctype *) bin->data;                                                \
+  old = scm_long2num ((long) data[idx]); data[idx] = (ctype) val;            \
+  return old;                                                                \
+}
+
+/* Returns the @code{long} value of the binary smob @var{binary} at the 
+   array index @var{index}. */
+#define FUNC_NAME "binary-long-ref"
+MAKE_BIN_REF (long)
+#undef FUNC_NAME
+
+/* Sets the @code{long} value of the binary smob @var{binary} at the array
+   index @var{index} to the given value @var{value}. The procedure returns
+   the previous (overridden) value. */
+#define FUNC_NAME "binary-long-set!"
+MAKE_BIN_SET (long)
+#undef FUNC_NAME
+
+/* Returns the @code{int} value of the binary smob @var{binary} at the 
+   array index @var{index}. */
+#define FUNC_NAME "binary-int-ref"
+MAKE_BIN_REF (int)
+#undef FUNC_NAME
+
+/* Sets the @code{int} value of the binary smob @var{binary} at the array
+   index @var{index} to the given value @var{value}. The procedure returns
+   the previous (overridden) value. */
+#define FUNC_NAME "binary-int-set!"
+MAKE_BIN_SET (int)
+#undef FUNC_NAME
+
+/* Returns the @code{short} value of the binary smob @var{binary} at the 
+   array index @var{index}. */
+#define FUNC_NAME "binary-short-ref"
+MAKE_BIN_REF (short)
+#undef FUNC_NAME
+
+/* Sets the @code{short} value of the binary smob @var{binary} at the array
+   index @var{index} to the given value @var{value}. The procedure returns
+   the previous (overridden) value. */
+#define FUNC_NAME "binary-short-set!"
+MAKE_BIN_SET (short)
+#undef FUNC_NAME
+
+/* Returns the @code{char} value of the binary smob @var{binary} at the 
+   array index @var{index}. */
+#define FUNC_NAME "binary-char-ref"
+MAKE_BIN_REF (char)
+#undef FUNC_NAME
+
+/* Sets the @code{char} value of the binary smob @var{binary} at the array
+   index @var{index} to the given value @var{value}. The procedure returns
+   the previous (overridden) value. */
+#define FUNC_NAME "binary-char-set!"
+MAKE_BIN_SET (char)
+#undef FUNC_NAME
+
 /* Initialize the binary smob with all its features. Call this function
    once at application startup and before running any scheme file 
    evaluation. */
 void
 guile_bin_init (void)
 {
+#if HAVE_OLD_SMOBS
+  /* Guile 1.3 backward compatibility code. */
+  static scm_smobfuns guile_bin_funs = {
+    NULL, guile_bin_free, guile_bin_print, guile_bin_equal };
+  guile_bin_tag = scm_newsmob (&guile_bin_funs);
+#else
+  /* Create new smob data type. */
   guile_bin_tag = scm_make_smob_type ("svz-binary", 0);
   scm_set_smob_print (guile_bin_tag, guile_bin_print);
   scm_set_smob_free (guile_bin_tag, guile_bin_free);
   scm_set_smob_equalp (guile_bin_tag, guile_bin_equal);
+#endif /* not HAVE_OLD_SMOBS */
 
   scm_c_define_gsubr ("binary?", 1, 0, 0, guile_bin_p);
   scm_c_define_gsubr ("string->binary", 1, 0, 0, guile_string_to_bin);
@@ -490,6 +599,15 @@ guile_bin_init (void)
   scm_c_define_gsubr ("binary-length", 1, 0, 0, guile_bin_length);
   scm_c_define_gsubr ("binary-concat!", 2, 0, 0, guile_bin_concat_x);
   scm_c_define_gsubr ("binary-subset", 2, 1, 0, guile_bin_subset);
+
+  scm_c_define_gsubr ("binary-long-ref", 2, 0, 0, guile_bin_long_ref);
+  scm_c_define_gsubr ("binary-int-ref", 2, 0, 0, guile_bin_int_ref);
+  scm_c_define_gsubr ("binary-short-ref", 2, 0, 0, guile_bin_short_ref);
+  scm_c_define_gsubr ("binary-char-ref", 2, 0, 0, guile_bin_char_ref);
+  scm_c_define_gsubr ("binary-long-set!", 3, 0, 0, guile_bin_long_set);
+  scm_c_define_gsubr ("binary-int-set!", 3, 0, 0, guile_bin_int_set);
+  scm_c_define_gsubr ("binary-short-set!", 3, 0, 0, guile_bin_short_set);
+  scm_c_define_gsubr ("binary-char-set!", 3, 0, 0, guile_bin_char_set);
 }
 
 #else /* not ENABLE_GUILE_SERVER */
