@@ -1,7 +1,7 @@
 /*
  * guile-server.c - guile server modules
  *
- * Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile-server.c,v 1.38 2001/11/27 14:21:33 ela Exp $
+ * $Id: guile-server.c,v 1.43 2002/02/10 11:38:28 ela Exp $
  *
  */
 
@@ -45,13 +45,13 @@
 #include "guile-bin.h"
 #include "guile-server.h"
 
-/* The guile server hash. */
+/* The guile server type hash. */
 static svz_hash_t *guile_server = NULL;
 
 /* The guile socket hash. */
 static svz_hash_t *guile_sock = NULL;
 
-/* List of functions to configure. */
+/* List of functions to configure for a server type. */
 static char *guile_functions[] = {
   "global-init", "init", "detect-proto", "connect-socket", "finalize",
   "global-finalize", "info-client", "info-server", "notify", 
@@ -61,7 +61,29 @@ static char *guile_functions[] = {
 static int guile_use_exceptions = 1;
 
 /*
- * Creates a Guile SMOB (small object). The @var{cfunc} specifies a base 
+ * Depending on the smob implementation of Guile we use different functions
+ * in order to create a new smob tag. It is also necessary to apply a smob
+ * `free' function for older Guile versions because it is called 
+ * unconditionally and has no reasonable default function.
+ */
+
+#if HAVE_OLD_SMOBS
+  /* Guile 1.3 backward compatibility code. */
+# define CREATE_SMOB_TAG(ctype, description)                               \
+  static scm_smobfuns guile_funs = {                                       \
+    NULL, GUILE_CONCAT3 (guile_,ctype,_free),                              \
+    GUILE_CONCAT3 (guile_,ctype,_print), NULL };                           \
+  GUILE_CONCAT3 (guile_,ctype,_tag) = scm_newsmob (&guile_funs);
+#else
+  /* Create new smob data type and assign a printer function. */
+# define CREATE_SMOB_TAG(ctype, description)                               \
+  GUILE_CONCAT3 (guile_,ctype,_tag) = scm_make_smob_type (description, 0); \
+  scm_set_smob_print (GUILE_CONCAT3 (guile_,ctype,_tag),                   \
+                      GUILE_CONCAT3 (guile_,ctype,_print));
+#endif
+
+/*
+ * Creates a Guile SMOB (small object). The @var{ctype} specifies a base 
  * name for all defined functions. The argument @var{description} is used
  * in the printer function. The macro creates various function to operate
  * on a SMOB:
@@ -69,99 +91,80 @@ static int guile_use_exceptions = 1;
  * b) getter  - Converts a scheme cell to the C structure.
  * c) checker - Checks if the scheme cell represents this SMOB.
  * d) printer - Used when applying (display . args) in Guile.
- * e) init    - Initialization of the SMOB.
+ * e) init    - Initialization of the SMOB type
  * f) tag     - The new scheme tag used to identify a SMOB.
+ * g) free    - Run if the SMOB gets destroyed.
  */
-#define MAKE_SMOB_DEFINITION(cfunc, description)                             \
-long guile_##cfunc##_tag = 0;                                                \
-SCM guile_##cfunc##_create (void *data) {                                    \
-  SCM value;                                                                 \
-  SCM_NEWSMOB (value, guile_##cfunc##_tag, data);                            \
-  return value;                                                              \
+#define MAKE_SMOB_DEFINITION(ctype, description)                             \
+static long GUILE_CONCAT3 (guile_,ctype,_tag) = 0;                           \
+static SCM GUILE_CONCAT3 (guile_,ctype,_create) (void *data) {               \
+  SCM_RETURN_NEWSMOB (GUILE_CONCAT3 (guile_,ctype,_tag), data);              \
 }                                                                            \
-void * guile_##cfunc##_get (SCM smob) {                                      \
+static void * GUILE_CONCAT3 (guile_,ctype,_get) (SCM smob) {                 \
   return (void *) SCM_SMOB_DATA (smob);                                      \
 }                                                                            \
-int guile_##cfunc##_p (SCM smob) {                                           \
+static int GUILE_CONCAT3 (guile_,ctype,_p) (SCM smob) {                      \
   return (SCM_NIMP (smob) &&                                                 \
-          SCM_TYP16 (smob) == guile_##cfunc##_tag) ? 1 : 0;                  \
+          SCM_TYP16 (smob) == GUILE_CONCAT3 (guile_,ctype,_tag)) ? 1 : 0;    \
 }                                                                            \
-int guile_##cfunc##_print (SCM smob, SCM port, scm_print_state *state) {     \
+static int GUILE_CONCAT3 (guile_,ctype,_print) (SCM smob, SCM port,          \
+                                                scm_print_state *state) {    \
   static char txt[256];                                                      \
   sprintf (txt, "#<%s %p>", description, (void *) SCM_SMOB_DATA (smob));     \
   scm_puts (txt, port);                                                      \
   return 1;                                                                  \
 }                                                                            \
-void guile_##cfunc##_init (void) {                                           \
-  guile_##cfunc##_tag = scm_make_smob_type (description, 0);                 \
-  scm_set_smob_print (guile_##cfunc##_tag, guile_##cfunc##_print);           \
+static scm_sizet GUILE_CONCAT3 (guile_,ctype,_free) (SCM smob) {             \
+  return 0;                                                                  \
+}                                                                            \
+static void GUILE_CONCAT3 (guile_,ctype,_init) (void) {                      \
+  CREATE_SMOB_TAG (ctype, description);                                      \
 }
 
-#define INIT_SMOB(cfunc) \
-  guile_##cfunc##_init ()
+/* Initializer macro for a new smob type. */
+#define INIT_SMOB(ctype) GUILE_CONCAT3 (guile_,ctype,_init) ()
 
-#define MAKE_SMOB(cfunc, data) \
-  guile_##cfunc##_create (data)
+/* Intanciating macro for a smob type. */
+#define MAKE_SMOB(ctype, data) GUILE_CONCAT3 (guile_,ctype,_create) (data)
 
-#define CHECK_SMOB(cfunc, smob) \
-  guile_##cfunc##_p (smob)
+/* Checks if the given scheme cell is a smob or not. */
+#define CHECK_SMOB(ctype, smob) GUILE_CONCAT3 (guile_,ctype,_p) (smob)
 
-#define CHECK_SMOB_ARG(cfunc, smob, arg, description, var)        \
-  do {                                                            \
-    if (!CHECK_SMOB (cfunc, smob))                                \
-      scm_wrong_type_arg_msg (FUNC_NAME, arg, smob, description); \
-    else                                                          \
-      var = GET_SMOB (cfunc, smob);                               \
+/* Extracts the smob data from a given smob cell. */
+#define GET_SMOB(ctype, smob) GUILE_CONCAT3 (guile_,ctype,_get) (smob)
+
+/* Checks if the scheme cell @var{smob} is a smob and throws an error if
+   not. Otherwise the variable @var{var} receives the smob data. */
+#define CHECK_SMOB_ARG(ctype, smob, arg, description, var) do { \
+  if (!CHECK_SMOB (ctype, smob))                                \
+    scm_wrong_type_arg_msg (FUNC_NAME, arg, smob, description); \
+  else                                                          \
+    var = GET_SMOB (ctype, smob);                               \
   } while (0)
 
-#define GET_SMOB(cfunc, smob) \
-  guile_##cfunc##_get (smob)
-
-#define INIT_SMOB_MARKER(cfunc) \
-  scm_set_smob_mark (guile_##cfunc##_tag, guile_##cfunc##_gc)
+/* Finally: With the help of the above macros we create smob types for
+   Serveez socket structures, servers and server types. */
+MAKE_SMOB_DEFINITION (svz_socket, "svz-socket")
+MAKE_SMOB_DEFINITION (svz_server, "svz-server")
+MAKE_SMOB_DEFINITION (svz_servertype, "svz-servertype")
 
 /* This macro creates a socket callback getter/setter for use in Guile.
    The procedure returns any previously set callback or an undefined value. */
 #define MAKE_SOCK_CALLBACK(func, assoc)                                \
-static SCM guile_sock_##func (SCM sock, SCM proc) {                    \
+static SCM GUILE_CONCAT2 (guile_sock_,func) (SCM sock, SCM proc) {     \
   svz_socket_t *xsock;                                                 \
   CHECK_SMOB_ARG (svz_socket, sock, SCM_ARG1, "svz-socket", xsock);    \
   if (!SCM_UNBNDP (proc)) {                                            \
     SCM_ASSERT_TYPE (SCM_PROCEDUREP (proc), proc, SCM_ARG2, FUNC_NAME, \
 		     "procedure");                                     \
-    xsock->##func = guile_func_##func;                                 \
+    GUILE_CONCAT2 (xsock->,func) = GUILE_CONCAT2 (guile_func_,func);   \
     return guile_sock_setfunction (xsock, assoc, proc); }              \
   return guile_sock_getfunction (xsock, assoc);                        \
 }
 
 /* Provides a socket callback setter/getter. */
 #define DEFINE_SOCK_CALLBACK(assoc, func) \
-  scm_c_define_gsubr (assoc, 1, 1, 0, guile_sock_##func)
-
-MAKE_SMOB_DEFINITION (svz_socket, "svz-socket")
-MAKE_SMOB_DEFINITION (svz_server, "svz-server")
-MAKE_SMOB_DEFINITION (svz_servertype, "svz-servertype")
-
-/* Garbage collector function for guile servertype definitions. */
-static SCM
-guile_svz_servertype_gc (SCM server)
-{
-  return SCM_BOOL_F;
-}
-
-/* Garbage collector function for guile socket structures. */
-static SCM
-guile_svz_socket_gc (SCM socket)
-{
-  return SCM_BOOL_F;
-}
-
-/* Garbage collector function for guile server definitions. */
-static SCM
-guile_svz_server_gc (SCM server)
-{
-  return SCM_BOOL_F;
-}
+  scm_c_define_gsubr (assoc, 1, 1, 0, GUILE_CONCAT2 (guile_sock_,func))
 
 /*
  * Extract a guile procedure from an option hash. Return zero on success.
@@ -278,6 +281,15 @@ guile_sock_getfunction (svz_socket_t *sock, char *func)
   return proc;
 }
 
+/* This function is used as destruction callback for the socket and
+ * servertype hashes used in the Guile servers. */
+static void
+guile_unprotect (SCM proc)
+{
+  if (!SCM_UNBNDP (proc))
+    scm_gc_unprotect_object (proc);
+}
+
 /*
  * Associate the given guile procedure @var{proc} hereby named @var{func}
  * with the socket structure @var{sock}. The function returns the previously
@@ -294,7 +306,7 @@ guile_sock_setfunction (svz_socket_t *sock, char *func, SCM proc)
 
   if ((gsock = svz_hash_get (guile_sock, svz_itoa (sock->id))) == NULL)
     {
-      gsock = svz_hash_create (4);
+      gsock = svz_hash_create (4, (svz_free_func_t) guile_unprotect);
       svz_hash_put (guile_sock, svz_itoa (sock->id), gsock);
     }
 
@@ -495,19 +507,6 @@ guile_func_detect_proto (svz_server_t *server, svz_socket_t *sock)
   return 0;
 }
 
-/* Destroys the given hash containing the guile procedure callbacks
-   associated with a socket structure. */
-static void
-guile_sock_clear_callbacks (svz_hash_t *callbacks)
-{
-  int n;
-  SCM *proc;
-
-  svz_hash_foreach_value (callbacks, proc, n)
-    scm_gc_unprotect_object (proc[n]);
-  svz_hash_destroy (callbacks);
-}
-
 /* Free the socket boundary if set by guile. */
 static void
 guile_sock_clear_boundary (svz_socket_t *sock)
@@ -539,7 +538,7 @@ guile_func_disconnected_socket (svz_socket_t *sock)
 
   /* Delete all the associated guile callbacks and unprotect these. */
   if ((gsock = svz_hash_delete (guile_sock, svz_itoa (sock->id))) != NULL)
-    guile_sock_clear_callbacks (gsock);
+    svz_hash_destroy (gsock);
 
   /* Release associated guile object is necessary. */
   if (sock->data != NULL)
@@ -592,9 +591,8 @@ guile_func_finalize (svz_server_t *server)
 {
   svz_servertype_t *stype = svz_servertype_find (server);
   SCM ret, finalize = guile_servertype_getfunction (stype, "finalize");
-  int i, retval = 0;
+  int retval = 0;
   svz_hash_t *state;
-  void **object;
 
   if (!SCM_UNBNDP (finalize))
     {
@@ -605,8 +603,6 @@ guile_func_finalize (svz_server_t *server)
   /* Release associated guile server state objects is necessary. */
   if ((state = server->data) != NULL)
     {
-      svz_hash_foreach_value (state, object, i)
-	scm_gc_unprotect_object ((SCM) SVZ_PTR2NUM (object[i]));
       svz_hash_destroy (state);
       server->data = NULL;
     }
@@ -861,7 +857,7 @@ guile_sock_print (SCM sock, SCM buffer)
       buf = guile_bin_to_data (buffer, &len);
     }
 
-  /* Depending on the protcol type use different kind of senders. */
+  /* Depending on the protocol type use different kind of senders. */
   if (xsock->proto & (PROTO_TCP | PROTO_PIPE))
     ret = svz_sock_write (xsock, buf, len);
   else if (xsock->proto & PROTO_UDP)
@@ -1045,7 +1041,7 @@ guile_server_state_set_x (SCM server, SCM key, SCM value)
 
   if ((hash = xserver->data) == NULL)
     {
-      hash = svz_hash_create (4);
+      hash = svz_hash_create (4, (svz_free_func_t) guile_unprotect);
       xserver->data = hash;
     }
   if ((val = svz_hash_put (hash, str, SVZ_NUM2PTR (value))) != NULL)
@@ -1074,7 +1070,7 @@ guile_server_state_to_hash (SCM server)
   CHECK_SERVER_SMOB_ARG (server, SCM_ARG1, xserver);
   if ((data = xserver->data) != NULL)
     {
-      hash = scm_make_vector (scm_int2num (svz_hash_size (data)), SCM_EOL);
+      hash = scm_c_make_vector (svz_hash_size (data), SCM_EOL);
       svz_hash_foreach_key (data, key, i)
 	scm_hash_set_x (hash, scm_makfrom0str (key[i]),
 			(SCM) SVZ_PTR2NUM (svz_hash_get (data, key[i])));
@@ -1489,7 +1485,7 @@ guile_define_servertype (SCM args)
 				    &server->description, txt);
 
   /* Set the procedures. */
-  functions = svz_hash_create (4);
+  functions = svz_hash_create (4, (svz_free_func_t) guile_unprotect);
   for (n = 0; guile_functions[n] != NULL; n++)
     {
       err |= optionhash_extract_proc (options, guile_functions[n],
@@ -1530,6 +1526,34 @@ guile_define_servertype (SCM args)
 }
 #undef FUNC_NAME
 
+/*
+ * Destroys the servertype represented by the hash @var{callbacks}. Removes
+ * the servertype pointer from the hash, destroys the remaining callback
+ * hash and finally frees all resources allocated by the servertype.
+ */
+static void
+guile_servertype_destroy (svz_hash_t *callbacks)
+{
+  svz_servertype_t *stype;
+
+  stype = svz_hash_delete (callbacks, "server-type");
+  svz_hash_destroy (callbacks);
+  svz_free (stype->prefix);
+  svz_free (stype->description);
+  guile_servertype_config_free (stype);
+  svz_free (stype);
+}
+
+/*
+ * Destroys the given socket represented by the hash @var{callbacks} and
+ * destroy this hash recursively.
+ */
+static void
+guile_sock_destroy (svz_hash_t *callbacks)
+{
+  svz_hash_destroy (callbacks);
+}
+
 #include "guile-api.c"
 
 /*
@@ -1543,8 +1567,10 @@ guile_server_init (void)
   if (guile_server || guile_sock)
     return;
 
-  guile_server = svz_hash_create (4);
-  guile_sock = svz_hash_create (4);
+  guile_server = 
+    svz_hash_create (4, (svz_free_func_t) guile_servertype_destroy);
+  guile_sock = 
+    svz_hash_create (4, (svz_free_func_t) guile_sock_destroy);
 
   scm_c_define_gsubr ("define-servertype!", 
 		      1, 0, 0, guile_define_servertype);
@@ -1568,8 +1594,8 @@ guile_server_init (void)
 		      0, 1, 0, guile_access_exceptions);
   scm_c_define_gsubr ("serveez-nuke",
 		      0, 0, 0, guile_nuke_happened);
-  DEFINE_SOCK_CALLBACK ("svz:sock:handle-request", handle_request);
-  DEFINE_SOCK_CALLBACK ("svz:sock:check-request", check_request);
+  DEFINE_SOCK_CALLBACK ("svz:sock:handle-request",handle_request);
+  DEFINE_SOCK_CALLBACK ("svz:sock:check-request",check_request);
 
   /* Initialize the guile SMOB things. Previously defined via 
      MAKE_SMOB_DEFINITION (). */
@@ -1577,34 +1603,10 @@ guile_server_init (void)
   INIT_SMOB (svz_server);
   INIT_SMOB (svz_servertype);
   
-  /* Setup garbage collector marking functions. */
-  INIT_SMOB_MARKER (svz_socket);
-  INIT_SMOB_MARKER (svz_server);
-  INIT_SMOB_MARKER (svz_servertype);
-
   guile_bin_init ();
   guile_api_init ();
 }
 
-/* Destroys the server type hash containing all callbacks associated with
-   the returned server type. */
-static svz_servertype_t *
-guile_servertype_clear_callbacks (svz_hash_t *callbacks)
-{
-  svz_servertype_t *server;
-  SCM *proc;
-  int n;
-
-  server = svz_hash_get (callbacks, "server-type");
-  svz_hash_foreach_value (callbacks, proc, n)
-    {
-      if (proc[n] != (SCM) server && !SCM_UNBNDP (proc[n]))
-	scm_gc_unprotect_object (proc[n]);
-    }
-  svz_hash_destroy (callbacks);
-  return server;
-}
-
 /*
  * This function should be called before shutting down the core library in
  * order to avoid memory leaks. It releases the server types defined with
@@ -1613,35 +1615,16 @@ guile_servertype_clear_callbacks (svz_hash_t *callbacks)
 void
 guile_server_finalize (void)
 {
-  char **prefix, **id;
-  int i;
-  svz_servertype_t *stype;
-  svz_hash_t *functions;
-
   guile_api_finalize ();
 
   if (guile_server)
     {
-      svz_hash_foreach_key (guile_server, prefix, i)
-	{
-	  functions = svz_hash_get (guile_server, prefix[i]);
-	  stype = guile_servertype_clear_callbacks (functions);
-	  svz_free (stype->prefix);
-	  svz_free (stype->description);
-	  guile_servertype_config_free (stype);
-	  svz_free (stype);
-	}
       svz_hash_destroy (guile_server);
       guile_server = NULL;
     }
 
   if (guile_sock)
     {
-      svz_hash_foreach_key (guile_sock, id, i)
-	{
-	  functions = svz_hash_get (guile_sock, id[i]);
-	  guile_sock_clear_callbacks (functions);
-	}
       svz_hash_destroy (guile_sock);
       guile_sock = NULL;
     }
