@@ -2,7 +2,7 @@
  * guile.c - interface to Guile core library
  *
  * Copyright (C) 2001 Raimund Jacob <raimi@lkcc.org>
- * Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile.c,v 1.53 2001/11/29 23:41:43 raimi Exp $
+ * $Id: guile.c,v 1.59 2002/01/06 13:18:52 ela Exp $
  *
  */
 
@@ -94,21 +94,22 @@ guile_value_create (SCM value)
   return v;
 }
 
+/*
+ * Create a fresh option-hash.
+ */
+svz_hash_t *
+optionhash_create (void)
+{
+  return svz_hash_create (4, svz_free);
+}
+
 /*
  * Destroy the given option-hash @var{options}.
  */
 void
 optionhash_destroy (svz_hash_t *options)
 {
-  guile_value_t **value;
-  int n;
-
-  if (options)
-    {
-      svz_hash_foreach_value (options, value, n)
-	svz_free (value[n]);
-      svz_hash_destroy (options);
-    }
+  svz_hash_destroy (options);
 }
 
 /*
@@ -134,7 +135,7 @@ guile_get_current_load_port (void)
   SCM p = scm_current_load_port ();
   if (!SCM_FALSEP (p) && SCM_PORTP (p))
     return p;
-  if (SCM_PORTP (guile_load_port))
+  if (!SCM_UNBNDP (guile_load_port) && SCM_PORTP (guile_load_port))
     return guile_load_port;
   return SCM_UNDEFINED;
 }
@@ -150,13 +151,13 @@ guile_error (char *format, ...)
   va_list args;
   /* FIXME: Why is this port undefined in guile exceptions ? */
   SCM lp = guile_get_current_load_port ();
-  char *file = 
-    SCM_PORTP (lp) ? gh_scm2newstr (SCM_FILENAME (lp), NULL) : NULL;
+  char *file = (!SCM_UNBNDP (lp) && SCM_PORTP (lp)) ? 
+    gh_scm2newstr (SCM_FILENAME (lp), NULL) : NULL;
 
   /* guile counts lines from 0, we have to add one */
   fprintf (stderr, "%s:%d:%d: ", file ? file : "undefined", 
-	   SCM_PORTP (lp) ? SCM_LINUM (lp) + 1 : 0, 
-	   SCM_PORTP (lp) ? SCM_COL (lp) : 0);
+	   (!SCM_UNBNDP (lp) && SCM_PORTP (lp)) ? SCM_LINUM (lp) + 1 : 0, 
+	   (!SCM_UNBNDP (lp) && SCM_PORTP (lp)) ? SCM_COL (lp) : 0);
   if (file)
     scm_c_free (file);
 
@@ -241,7 +242,7 @@ optionhash_get (svz_hash_t *hash, char *key)
 svz_hash_t *
 guile_to_optionhash (SCM pairlist, char *txt, int dounpack)
 {
-  svz_hash_t *hash = svz_hash_create (10);
+  svz_hash_t *hash = optionhash_create ();
   guile_value_t *old_value;
   guile_value_t *new_value;
   int err = 0;
@@ -401,7 +402,8 @@ guile_to_hash (SCM list, char *prefix)
     }
 
   /* Iterate the alist. */
-  hash = svz_hash_create (SCM_NUM2ULONG (SCM_ARG1, scm_length (list)));
+  hash = svz_hash_create (SCM_NUM2ULONG (SCM_ARG1, scm_length (list)), 
+			  svz_free);
   for (i = 0; SCM_PAIRP (list); list = SCM_CDR (list), i++)
     {
       SCM k, v, pair = SCM_CAR (list);
@@ -456,9 +458,6 @@ guile_to_hash (SCM list, char *prefix)
   /* Free the values, keys are freed be hash destructor. */
   if (err)
     {
-      char **values;
-      svz_hash_foreach_value (hash, values, i)
-	svz_free (values[i]);
       svz_hash_destroy (hash);
       return NULL;
     }
@@ -883,7 +882,7 @@ optionhash_cb_hash (char *server, void *arg, char *key,
 	}
 
       /* Iterate the alist. */
-      hash = svz_hash_create (7);
+      hash = svz_hash_create (7, svz_free);
       for (i = 0; SCM_PAIRP (hvalue); hvalue = SCM_CDR (hvalue), i++)
 	{
 	  SCM k, v, pair = SCM_CAR (hvalue);
@@ -940,9 +939,6 @@ optionhash_cb_hash (char *server, void *arg, char *key,
       /* Free the values, keys are freed be hash destructor. */
       if (err)
 	{
-	  char **values;
-	  svz_hash_foreach_value (hash, values, i)
-	    svz_free (values[i]);
 	  svz_hash_destroy (hash);
 	  return SVZ_ITEM_FAILED;
 	}
@@ -1081,7 +1077,7 @@ guile_define_server (SCM name, SCM args)
 
   /* Extract options if any. */
   if (SCM_UNBNDP (args))
-    options = svz_hash_create (4);
+    options = optionhash_create ();
   else if (NULL == (options = guile_to_optionhash (args, txt, 0)))
     FAIL (); /* Message already emitted. */
 
@@ -1144,6 +1140,12 @@ guile_define_server (SCM name, SCM args)
 }
 #undef FUNC_NAME
 
+/* Validate a network port value. */
+#define GUILE_VALIDATE_PORT(port, name, proto) do {                         \
+  if ((port) < 0 || (port) > 65535) {                                       \
+    guile_error ("%s: %s port requires a short (0..65535)\n", proto, name); \
+    err = -1; } } while (0)
+
 /*
  * Port configuration definition. Use two arguments:
  * First is a (unique) name for the port configuration. Second is an 
@@ -1195,11 +1197,14 @@ guile_define_port (SCM name, SCM args)
       int port;
       cfg->proto = PROTO_TCP;
       err |= optionhash_extract_int (options, PORTCFG_PORT, 0, 0, &port, txt);
-      cfg->tcp_port = (short) port;
+      GUILE_VALIDATE_PORT (port, "TCP", portname);
+      cfg->tcp_port = (unsigned short) port;
       err |= optionhash_extract_int (options, PORTCFG_BACKLOG, 1, 0, 
 				     &(cfg->tcp_backlog), txt);
       err |= optionhash_extract_string (options, PORTCFG_IP, 1, PORTCFG_NOIP,
 					&(cfg->tcp_ipaddr), txt);
+      err |= optionhash_extract_string (options, PORTCFG_DEVICE, 1, NULL,
+					&(cfg->tcp_device), txt);
     }
   /* Maybe UDP ? */
   else if (!strcmp (proto, PORTCFG_UDP))
@@ -1207,9 +1212,12 @@ guile_define_port (SCM name, SCM args)
       int port;
       cfg->proto = PROTO_UDP;
       err |= optionhash_extract_int (options, PORTCFG_PORT, 0, 0, &port, txt);
-      cfg->udp_port = (short) port;
+      GUILE_VALIDATE_PORT (port, "UDP", portname);
+      cfg->udp_port = (unsigned short) port;
       err |= optionhash_extract_string (options, PORTCFG_IP, 1, PORTCFG_NOIP,
 					&(cfg->udp_ipaddr), txt);
+      err |= optionhash_extract_string (options, PORTCFG_DEVICE, 1, NULL,
+					&(cfg->udp_device), txt);
     }
   /* Maybe ICMP ? */
   else if (!strcmp (proto, PORTCFG_ICMP))
@@ -1218,6 +1226,8 @@ guile_define_port (SCM name, SCM args)
       cfg->proto = PROTO_ICMP;
       err |= optionhash_extract_string (options, PORTCFG_IP, 1, PORTCFG_NOIP,
 					&(cfg->icmp_ipaddr), txt);
+      err |= optionhash_extract_string (options, PORTCFG_DEVICE, 1, NULL,
+					&(cfg->icmp_device), txt);
       err |= optionhash_extract_int (options, PORTCFG_TYPE, 1, ICMP_SERVEEZ, 
 				     &type, txt);
       if (type & ~0xff)
@@ -1234,6 +1244,8 @@ guile_define_port (SCM name, SCM args)
       cfg->proto = PROTO_RAW;
       err |= optionhash_extract_string (options, PORTCFG_IP, 1, PORTCFG_NOIP,
 					&(cfg->raw_ipaddr), txt);
+      err |= optionhash_extract_string (options, PORTCFG_DEVICE, 1, NULL,
+					&(cfg->raw_device), txt);
     }
   /* Finally a PIPE ? */
   else if (!strcmp (proto, PORTCFG_PIPE))
@@ -1528,7 +1540,7 @@ guile_access_interfaces (SCM args)
 		  continue;
 		}
 	      sprintf (description, "guile interface %d", n);
-	      svz_interface_add (n, description, addr.sin_addr.s_addr);
+	      svz_interface_add (n, description, addr.sin_addr.s_addr, 0);
 	    }
 	  svz_array_destroy (array);
 	}
@@ -1752,9 +1764,8 @@ guile_load_config (char *cfgfile)
   if (SCM_FALSEP (ret))
     guile_global_error = -1; 
 
-  /* Kick the gc now since no guile is involved anymore unless a guile
-   * server is implemented.
-   */
+  /* Kick the garbage collection now since no Guile is involved anymore 
+     unless a Guile server is implemented. */
   scm_gc ();
 
   return guile_global_error ? -1 : 0;
