@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: http-cgi.h,v 1.9 2001/05/19 23:04:57 ela Exp $
+ * $Id: http-cgi.h,v 1.12 2001/08/03 18:09:04 ela Exp $
  *
  */
 
@@ -33,19 +33,10 @@
 
 #include "http-proto.h"
 
-#define MAX_CGI_DIR_LEN  256 /* length of a cgi script directory */
-
 #define POST_METHOD 0            /* POST id */
 #define GET_METHOD  1            /* GET id */
 #define HTTP_NO_CGI ((char *)-1) /* 'no cgi' pointer */
 
-#ifdef __MINGW32__
-# define ENV_BLOCK_SIZE 2048 /* max. environment block size */
-#else
-# define ENV_LENGTH     256  /* length of an environment variable */
-# define ENV_ENTRIES    64   /* max. amount of environment variables */
-#endif
-
 #define CGI_VERSION "CGI/1.0"
 
 char *http_check_cgi (svz_socket_t *sock, char *request);
@@ -57,6 +48,5 @@ int http_cgi_read (svz_socket_t *sock);
 int http_cgi_disconnect (svz_socket_t *sock);
 int http_cgi_died (svz_socket_t *sock);
 void http_gen_cgi_apps (http_config_t *cfg);
-void http_free_cgi_apps (http_config_t *cfg);
 
 #endif /* __HTTP_CGI_H__ */
