@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: http-dirlist.c,v 1.20 2001/04/01 13:32:29 ela Exp $
+ * $Id: http-dirlist.c,v 1.23 2001/12/23 16:28:39 ela Exp $
  *
  */
 
@@ -57,7 +57,7 @@
 # endif
 #endif /* not __MINGW32__ */
 
-#if HAVE_SYS_DIRENT_H
+#if HAVE_SYS_DIRENT_H && !defined (HAVE_DIRENT_H)
 # include <sys/dirent.h>
 #endif
 
@@ -86,6 +86,10 @@
 # define closedir(dir) FindClose (dir)
 #endif
 
+#if defined (HAVE_ALPHASORT) && !defined (DECLARED_ALPHASORT)
+extern int alphasort (const struct dirent **, const struct dirent **);
+#endif
+
 /* Size of last buffer allocated. */
 int http_dirlist_size = 0;
 
