@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: irc-event-2.c,v 1.12 2000/12/18 18:28:35 ela Exp $
+ * $Id: irc-event-2.c,v 1.16 2001/05/19 23:04:57 ela Exp $
  *
  */
 
@@ -33,13 +33,10 @@
 #include <stdlib.h>
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 #endif
 
-#include "socket.h"
-#include "alloc.h"
-#include "util.h"
-#include "server-core.h"
+#include <libserveez.h>
 #include "irc-core/irc-core.h"
 #include "irc-proto.h"
 #include "irc-event.h"
@@ -51,13 +48,13 @@
  *                  ERR_NOTONCHANNEL
  */
 int
-irc_part_callback (socket_t sock, 
+irc_part_callback (svz_socket_t *sock, 
 		   irc_client_t *client, irc_request_t *request)
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *cl;
   irc_channel_t *channel;
-  socket_t xsock;
+  svz_socket_t *xsock;
   int n;
 
   /* do you have enough paras ? */
@@ -122,7 +119,7 @@ irc_client_banned (irc_client_t *client, irc_ban_t *ban)
  * Send the channel topic to an IRC client.
  */
 static void
-irc_channel_topic (socket_t sock,
+irc_channel_topic (svz_socket_t *sock,
 		   irc_client_t *client, irc_channel_t *channel)
 {
   irc_config_t *cfg = sock->cfg;
@@ -151,7 +148,7 @@ irc_channel_topic (socket_t sock,
  * Send a channels users to a specified client.
  */
 static void
-irc_channel_users (socket_t sock,
+irc_channel_users (svz_socket_t *sock,
 		   irc_client_t *client, 
 		   irc_channel_t *channel)
 {
@@ -195,13 +192,13 @@ irc_channel_users (socket_t sock,
  *                  RPL_TOPIC
  */
 int
-irc_join_callback (socket_t sock, 
+irc_join_callback (svz_socket_t *sock, 
 		   irc_client_t *client, irc_request_t *request)
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *cl;
   irc_channel_t *channel;
-  socket_t xsock;
+  svz_socket_t *xsock;
   char *chan;
   int n, i;
 
@@ -250,13 +247,13 @@ irc_join_callback (socket_t sock,
 		  if (--channel->invites != 0)
 		    {
 		      channel->invite[i] = channel->invite[channel->invites];
-		      channel->invite = xrealloc (channel->invite,
-						  sizeof (irc_client_t *) *
-						  channel->invites);
+		      channel->invite = svz_realloc (channel->invite,
+						     sizeof (irc_client_t *) *
+						     channel->invites);
 		    }
 		  else
 		    {
-		      xfree (channel->invite);
+		      svz_free (channel->invite);
 		      channel->invite = NULL;
 		    }
 		}
@@ -345,7 +342,7 @@ irc_ban_string (irc_ban_t *ban)
  * zero value if the client is an operator otherwise zero.
  */
 static int
-irc_client_oper (socket_t sock, 
+irc_client_oper (svz_socket_t *sock, 
 		 irc_client_t *client, irc_channel_t *channel, int flag)
 {
   irc_config_t *cfg = sock->cfg;
@@ -365,16 +362,15 @@ irc_client_oper (socket_t sock,
  */
 static int
 irc_client_flag (irc_client_t *client,   /* client changing the flag */
-		 socket_t sock,          /* this clients connection */
+		 svz_socket_t *sock,     /* this clients connection */
 		 int cflag,              /* this clients flags */
 		 char *nick,             /* the nick the mode is for */
 		 irc_channel_t *channel, /* mode change for this channel */
 		 int flag,               /* flag to be set / unset */
 		 char set)               /* set / unset */
 {
-  irc_config_t *cfg = sock->cfg;
   irc_client_t *cl;
-  socket_t xsock;
+  svz_socket_t *xsock;
   int i, n;
   unsigned l;
   static char *Modes = CHANNEL_MODES;
@@ -428,7 +424,7 @@ irc_client_flag (irc_client_t *client,   /* client changing the flag */
  */
 static int
 irc_user_flag (irc_client_t *client,   /* client changing the flag */
-	       socket_t sock,          /* this clients connection */
+	       svz_socket_t *sock,     /* this clients connection */
 	       int flag,               /* flag to be set / unset */
 	       char set)               /* set / unset */
 {
@@ -478,15 +474,14 @@ irc_user_flag (irc_client_t *client,   /* client changing the flag */
  */
 static int
 irc_channel_flag (irc_client_t *client,   /* client changing the flag */
-		  socket_t sock,          /* this clients connection */
+		  svz_socket_t *sock,     /* this clients connection */
 		  int cflag,              /* this clients flags */
 		  irc_channel_t *channel, /* mode change for this channel */
 		  int flag,               /* flag to be set / unset */
 		  char set)               /* set / unset */
 {
-  irc_config_t *cfg = sock->cfg;
   irc_client_t *cl;
-  socket_t xsock;
+  svz_socket_t *xsock;
   int n;
   unsigned l;
   char *Modes = CHANNEL_MODES;
@@ -522,18 +517,18 @@ irc_channel_flag (irc_client_t *client,   /* client changing the flag */
 }
 
 /*
- * Destroy and xfree() a given ban entry.
+ * Destroy and svz_free() a given ban entry.
  */
 void
 irc_destroy_ban (irc_ban_t *ban)
 {
   if (ban->nick)
-    xfree (ban->nick);
+    svz_free (ban->nick);
   if (ban->user)
-    xfree (ban->user);
-  xfree (ban->host);
-  xfree (ban->by);
-  xfree (ban);
+    svz_free (ban->user);
+  svz_free (ban->host);
+  svz_free (ban->by);
+  svz_free (ban);
 }
 
 /*
@@ -547,14 +542,14 @@ irc_create_ban (irc_client_t *client, char *request, int len)
   int n, size = 0;
 
   /* reserve and initialize buffer space */
-  ban = xmalloc (sizeof (irc_ban_t));
+  ban = svz_malloc (sizeof (irc_ban_t));
   memset (ban, 0, sizeof (irc_ban_t));
-  tmp = xmalloc (MAX_MSG_LEN);
+  tmp = svz_malloc (MAX_MSG_LEN);
   memset (tmp, 0, MAX_MSG_LEN);
 
   ban->since = time (NULL);
   sprintf (tmp, "%s!%s@%s", client->nick, client->user, client->host);
-  ban->by = xstrdup (tmp);
+  ban->by = svz_strdup (tmp);
 
   p = request;
   n = 0;
@@ -567,7 +562,7 @@ irc_create_ban (irc_client_t *client, char *request, int len)
   if (*p == '!')
     {
       tmp[n] = '\0';
-      ban->nick = xstrdup (tmp);
+      ban->nick = svz_strdup (tmp);
       p++;
       size++;
       n = 0;
@@ -580,7 +575,7 @@ irc_create_ban (irc_client_t *client, char *request, int len)
       if (*p == '@')
 	{
 	  tmp[n] = '\0';
-	  ban->user = xstrdup (tmp);
+	  ban->user = svz_strdup (tmp);
 	  p++;
 	  size++;
 	  n = 0;
@@ -590,14 +585,14 @@ irc_create_ban (irc_client_t *client, char *request, int len)
 	      size++;
 	    }
 	  tmp[n] = '\0';
-	  ban->host = xstrdup (tmp);
+	  ban->host = svz_strdup (tmp);
 	}
     }
   /* here user parsed without nick */
   else if (*p == '@')
     {
       tmp[n] = '\0';
-      ban->user = xstrdup (tmp);
+      ban->user = svz_strdup (tmp);
       p++;
       size++;
       n = 0;
@@ -607,16 +602,16 @@ irc_create_ban (irc_client_t *client, char *request, int len)
 	  size++;
 	}
       tmp[n] = '\0';
-      ban->host = xstrdup (tmp);
+      ban->host = svz_strdup (tmp);
     }
   /* parsed just a host without user and nick */
   else
     {
       tmp[n] = '\0';
-      ban->host = xstrdup (tmp);
+      ban->host = svz_strdup (tmp);
     }
 
-  xfree (tmp);
+  svz_free (tmp);
   return ban;
 }
 
@@ -625,7 +620,7 @@ irc_create_ban (irc_client_t *client, char *request, int len)
  */
 static int
 irc_channel_arg (irc_client_t *client,   /* client changing the flag */
-		 socket_t sock,          /* this clients connection */
+		 svz_socket_t *sock,     /* this clients connection */
 		 int cflag,              /* this clients flags */
 		 irc_channel_t *channel, /* mode change for this channel */
 		 int flag,               /* flag to be set / unset */
@@ -634,7 +629,7 @@ irc_channel_arg (irc_client_t *client,   /* client changing the flag */
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *cl;
-  socket_t xsock;
+  svz_socket_t *xsock;
   int n;
   unsigned l;
   char *Modes = CHANNEL_MODES;
@@ -673,8 +668,8 @@ irc_channel_arg (irc_client_t *client,   /* client changing the flag */
 	      return 0;
 	    }
 	  if (channel->key)
-	    xfree (channel->key);
-	  channel->key = xstrdup (arg);
+	    svz_free (channel->key);
+	  channel->key = svz_strdup (arg);
 	  channel->flag |= flag;
 	  break;
 	case MODE_ULIMIT:
@@ -683,8 +678,8 @@ irc_channel_arg (irc_client_t *client,   /* client changing the flag */
 	  break;
 	case MODE_BAN:
 	  n = channel->bans;
-	  channel->ban = xrealloc (channel->ban, 
-				   sizeof (irc_ban_t *) * (n + 1));
+	  channel->ban = svz_realloc (channel->ban, 
+				      sizeof (irc_ban_t *) * (n + 1));
 	  channel->ban[n] = irc_create_ban (client, arg, strlen (arg));
 	  channel->bans++;
 	  break;
@@ -704,13 +699,13 @@ irc_channel_arg (irc_client_t *client,   /* client changing the flag */
 		  if (--channel->bans != 0)
 		    {
 		      channel->ban[n] = channel->ban[channel->bans];
-		      channel->ban = xrealloc (channel->ban,
-					       sizeof (irc_ban_t *) *
-					       channel->bans);
+		      channel->ban = svz_realloc (channel->ban,
+						  sizeof (irc_ban_t *) *
+						  channel->bans);
 		    }
 		  else
 		    {
-		      xfree (channel->ban);
+		      svz_free (channel->ban);
 		      channel->ban = NULL;
 		    }
 		  break;
@@ -811,7 +806,7 @@ irc_channel_flag_string (irc_channel_t *channel)
 #define FLAG_UNSET 0
 
 int
-irc_mode_callback (socket_t sock, 
+irc_mode_callback (svz_socket_t *sock, 
 		   irc_client_t *client, irc_request_t *request)
 {
   irc_config_t *cfg = sock->cfg;
@@ -1022,13 +1017,13 @@ irc_mode_callback (socket_t sock,
  *                  ERR_CHANOPRIVSNEEDED
  */
 int
-irc_topic_callback (socket_t sock, 
+irc_topic_callback (svz_socket_t *sock, 
 		    irc_client_t *client, irc_request_t *request)
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *cl;
   irc_channel_t *channel;
-  socket_t xsock;
+  svz_socket_t *xsock;
   int n, i;
 
   /* enough paras ? */
@@ -1054,11 +1049,11 @@ irc_topic_callback (socket_t sock,
 
 	      /* change the topic */
 	      if (channel->topic)
-		xfree (channel->topic);
-	      channel->topic = xstrdup (request->para[1]);
+		svz_free (channel->topic);
+	      channel->topic = svz_strdup (request->para[1]);
 	      if (channel->topic_by)
-		xfree (channel->topic_by);
-	      channel->topic_by = xstrdup (client->nick);
+		svz_free (channel->topic_by);
+	      channel->topic_by = svz_strdup (client->nick);
 	      channel->topic_since = time (NULL);
 
 	      /* send topic to all clients in channel */
@@ -1082,7 +1077,7 @@ irc_topic_callback (socket_t sock,
  *   Numerics: RPL_NAMREPLY  RPL_ENDOFNAMES
  */
 int
-irc_names_callback (socket_t sock, 
+irc_names_callback (svz_socket_t *sock, 
 		    irc_client_t *client, irc_request_t *request)
 {
   irc_config_t *cfg = sock->cfg;
@@ -1096,9 +1091,9 @@ irc_names_callback (socket_t sock,
   if (request->paras < 1)
     {
       /* go through all channels */
-      if ((ch = (irc_channel_t **) hash_values (cfg->channels)) != NULL)
+      if ((ch = (irc_channel_t **) svz_hash_values (cfg->channels)) != NULL)
 	{
-	  for (n = 0; n < hash_size (cfg->channels); n++)
+	  for (n = 0; n < svz_hash_size (cfg->channels); n++)
 	    {
 	      /* 
 	       * you can't see secret and private channels, 
@@ -1111,7 +1106,7 @@ irc_names_callback (socket_t sock,
 	      /* send a line */
 	      irc_channel_users (sock, client, ch[n]);
 	    }
-	  hash_xfree (ch);
+	  svz_hash_xfree (ch);
 	}
 
       /* 
@@ -1120,9 +1115,9 @@ irc_names_callback (socket_t sock,
        * Public channels are not secret or private or channels
        * clients share.
        */
-      if ((cl = (irc_client_t **) hash_values (cfg->clients)) != NULL)
+      if ((cl = (irc_client_t **) svz_hash_values (cfg->clients)) != NULL)
 	{
-	  for (n = 0; n < hash_size (cfg->clients); n++)
+	  for (n = 0; n < svz_hash_size (cfg->clients); n++)
 	    {
 	      text[0] = 0;
 
@@ -1148,7 +1143,7 @@ irc_names_callback (socket_t sock,
 		    }
 		}
 	    }
-	  hash_xfree (cl);
+	  svz_hash_xfree (cl);
 	}
       /* send the (*) reply */
       irc_printf (sock, ":%s %03d %s " RPL_NAMREPLY_TEXT "\n",
@@ -1183,7 +1178,7 @@ irc_names_callback (socket_t sock,
  * Send a part of a channel list.
  */
 static void
-irc_channel_list (socket_t sock, 
+irc_channel_list (svz_socket_t *sock, 
 		  irc_client_t *client, irc_channel_t *channel)
 {
   irc_config_t *cfg = sock->cfg;
@@ -1222,7 +1217,7 @@ irc_channel_list (socket_t sock,
  *                  RPL_LIST         RPL_LISTEND
  */
 int
-irc_list_callback (socket_t sock, 
+irc_list_callback (svz_socket_t *sock, 
 		   irc_client_t *client,
 		   irc_request_t *request)
 {
@@ -1237,13 +1232,13 @@ irc_list_callback (socket_t sock,
   /* list all channels */
   if (request->paras < 1)
     {
-      if ((ch = (irc_channel_t **) hash_values (cfg->channels)) != NULL)
+      if ((ch = (irc_channel_t **) svz_hash_values (cfg->channels)) != NULL)
 	{
-	  for (n = 0; n < hash_size (cfg->channels); n++)
+	  for (n = 0; n < svz_hash_size (cfg->channels); n++)
 	    {
 	      irc_channel_list (sock, client, ch[n]);
 	    }
-	  hash_xfree (ch);
+	  svz_hash_xfree (ch);
 	}
     }
   /* list specified channels */
@@ -1274,13 +1269,13 @@ irc_list_callback (socket_t sock,
  *                  RPL_INVITING         RPL_AWAY
  */
 int
-irc_invite_callback (socket_t sock, 
+irc_invite_callback (svz_socket_t *sock, 
 		     irc_client_t *client, irc_request_t *request)
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *cl;
   irc_channel_t *ch;
-  socket_t xsock;
+  svz_socket_t *xsock;
   char *nick, *channel;
   int n, i;
 
@@ -1333,7 +1328,7 @@ irc_invite_callback (socket_t sock,
 
   /* fill in the invited client into the channels invite array */
   n = ch->invites;
-  ch->invite = xrealloc (ch->invite, sizeof (irc_client_t *) * (n + 1));
+  ch->invite = svz_realloc (ch->invite, sizeof (irc_client_t *) * (n + 1));
   ch->invite[n] = cl;
   ch->invites++;
 
@@ -1348,14 +1343,14 @@ irc_invite_callback (socket_t sock,
  *                  ERR_NOTONCHANNEL
  */
 int
-irc_kick_callback (socket_t sock, 
+irc_kick_callback (svz_socket_t *sock, 
 		   irc_client_t *client, irc_request_t *request)
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *cl;
   irc_client_t *victim;
   irc_channel_t *channel;
-  socket_t xsock;
+  svz_socket_t *xsock;
   int i, n;
 
   /* enough paras ? */
