@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: irc-server.h,v 1.7 2001/09/27 15:47:36 ela Exp $
+ * $Id: irc-server.h,v 1.8 2001/12/13 18:00:00 ela Exp $
  *
  */
 
@@ -32,7 +32,7 @@
 /*
  * These functions are exported from this IRC server module.
  */
-int irc_parse_line __P ((char *line, char *fmt, ...));
+int irc_parse_line __PARAMS ((char *line, char *fmt, ...));
 void irc_delete_servers (irc_config_t *cfg);
 void irc_connect_servers (irc_config_t *cfg);
 int irc_count_servers (irc_config_t *cfg);
