@@ -1,7 +1,7 @@
 /*
  * coserver.c - basic internal coserver routines
  *
- * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: coserver.c,v 1.23 2001/11/23 13:18:39 ela Exp $
+ * $Id: coserver.c,v 1.26 2002/02/15 12:16:07 ela Exp $
  *
  */
 
@@ -486,8 +486,9 @@ svz_coserver_check_request (svz_socket_t *sock)
   char *p = packet;
   int request_len;
   int len = 0;
-  svz_coserver_t *coserver;
+  svz_coserver_t *coserver = sock->data;
 
+  assert (coserver);
   do
     {
       /* find a line (trailing '\n') */
@@ -498,8 +499,6 @@ svz_coserver_check_request (svz_socket_t *sock)
       if (*p == COSERVER_PACKET_BOUNDARY && 
 	  p < sock->recv_buffer + sock->recv_buffer_fill)
 	{
-	  coserver = sock->data;
-	  assert (coserver);
 	  coserver->busy--;
 	  p++;
 	  request_len = p - packet;
@@ -512,7 +511,8 @@ svz_coserver_check_request (svz_socket_t *sock)
   while (p < sock->recv_buffer + sock->recv_buffer_fill);
       
 #if ENABLE_DEBUG
-  svz_log (LOG_DEBUG, "coserver: %d byte response\n", len);
+  svz_log (LOG_DEBUG, "%s: %d byte response\n", 
+	   svz_coservertypes[coserver->type].name, len);
 #endif
 
   /* remove data from receive buffer if necessary */
@@ -617,10 +617,13 @@ svz_coserver_close_pipes (svz_coserver_t *self)
 
 /* 
  * Iterate each socket object and close its file/socket/pipe 
- * descriptors.
+ * descriptors. Also frees the (cloned) queues.
+ * Note: Duplicate memory for everything else, including server private data.
+ *       We cannot take care of all that because the servers do not know
+ *       that they may get cloned. We therefore waste memory in the coservers.
  */
 static void
-svz_coserver_closeall (void)
+svz_coserver_closeall (svz_socket_t *self)
 {
   svz_socket_t *sock;
 
@@ -637,6 +640,11 @@ svz_coserver_closeall (void)
 	  if (sock->pipe_desc[WRITE] >= 2)
 	    close (sock->pipe_desc[WRITE]);
 	}
+      if (sock != self)
+	{
+	  svz_sock_resize_buffers (sock, 0, 0);
+	  svz_free (sock);
+	}
     }
   svz_file_closeall ();
 }
@@ -846,7 +854,7 @@ svz_coserver_start (int type)
 
       /* close all other coserver pipes except its own */
       svz_coserver_close_pipes (coserver);
-      svz_coserver_closeall ();
+      svz_coserver_closeall (coserver->sock);
       svz_coserver_signals ();
 
       /* start the internal coserver */
@@ -957,7 +965,8 @@ svz_coserver_check (void)
       ctype = &svz_coservertypes[coserver->type];
       sock = coserver->sock;
       if (sock->send_buffer_fill * 100 / sock->send_buffer_size >= 75 &&
-	  ((long) time (NULL)) - ctype->last_start >= 3)
+	  ((long) time (NULL)) - ctype->last_start >= 3 &&
+	  svz_coserver_count (ctype->type) <= ctype->instances)
 	svz_coserver_start (coserver->type);
     }
 }
@@ -983,7 +992,7 @@ svz_coserver_init (void)
   int i, n;
   svz_coservertype_t *coserver;
 
-  svz_coserver_callbacks = svz_hash_create (4);
+  svz_coserver_callbacks = svz_hash_create (4, svz_free);
   svz_coserver_callback_id = 1;
 
   for (n = 0; n < MAX_COSERVER_TYPES; n++)
@@ -1005,7 +1014,6 @@ int
 svz_coserver_finalize (void)
 {
   int n;
-  svz_coserver_callback_t **cb;
   svz_coservertype_t *coserver;
 
   for (n = 0; n < MAX_COSERVER_TYPES; n++)
@@ -1014,13 +1022,12 @@ svz_coserver_finalize (void)
       svz_coserver_destroy (coserver->type);
     }
 
-  /* @code{svz_free()} all callbacks left so far. */
-  svz_hash_foreach_value (svz_coserver_callbacks, cb, n)
-    svz_free (cb[n]);
 #if ENABLE_DEBUG
   svz_log (LOG_DEBUG, "coserver: %d callback(s) left\n",
 	   svz_hash_size (svz_coserver_callbacks));
 #endif
+
+  /* Destroy all callbacks left so far. */
   svz_hash_destroy (svz_coserver_callbacks);
   return 0;
 }
