@@ -1,7 +1,7 @@
 /*
  * portcfg.c - port configuration implementation
  *
- * Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: portcfg.c,v 1.25 2001/11/24 00:08:50 ela Exp $
+ * $Id: portcfg.c,v 1.34 2002/02/03 09:34:04 ela Exp $
  *
  */
 
@@ -68,42 +68,85 @@ svz_portcfg_create (void)
 
 /*
  * Check if two given port configurations structures are equal i.e. 
- * specifying the same network port or files. Returns non-zero if @var{a} 
- * and @var{b} are equal.
+ * specifying the same network port or pipe files. Returns 
+ * @code{PORTCFG_EQUAL} if @var{a} and @var{b} are identical, 
+ * @code{PORTCFG_MATCH} if the network address ofeither port 
+ * configurations contains the other (INADDR_ANY match) and otherwise 
+ * @code{PORTCFG_NOMATCH} or possibly @code{PORTCFG_CONFLICT}.
  */
 int
 svz_portcfg_equal (svz_portcfg_t *a, svz_portcfg_t *b)
 {
+  struct sockaddr_in *a_addr, *b_addr;
+
   if ((a->proto & (PROTO_TCP | PROTO_UDP | PROTO_ICMP | PROTO_RAW)) &&
       (a->proto == b->proto))
     {
-      /* 
-       * Two network ports are equal if both local port and address 
-       * are equal.
-       */
+      /* Two network ports are equal if both local port and IP address 
+	 are equal or one of them is INADDR_ANY. */
+      a_addr = svz_portcfg_addr (a);
+      b_addr = svz_portcfg_addr (b);
+
       switch (a->proto)
 	{
-	case PROTO_TCP:
-	  if (a->tcp_port == b->tcp_port &&
-	      !strcmp (a->tcp_ipaddr, b->tcp_ipaddr))
-	    return 1;
-	  break;
 	case PROTO_UDP:
-	  if (a->udp_port == b->udp_port &&
-	      !strcmp (a->udp_ipaddr, b->udp_ipaddr))
-	    return 1;
+	case PROTO_TCP:
+	  if (a_addr->sin_port == b_addr->sin_port)
+	    {
+	      if ((a->flags & PORTCFG_FLAG_DEVICE) || 
+		  (b->flags & PORTCFG_FLAG_DEVICE))
+		{
+		  if ((a->flags & PORTCFG_FLAG_DEVICE) && 
+		      (b->flags & PORTCFG_FLAG_DEVICE))
+		    {
+		      if (!strcmp (svz_portcfg_device (a), 
+				   svz_portcfg_device (b)))
+			return PORTCFG_EQUAL;
+		      return PORTCFG_NOMATCH;
+		    }
+		  return PORTCFG_CONFLICT;
+		}
+	      if (a_addr->sin_addr.s_addr == b_addr->sin_addr.s_addr)
+		return PORTCFG_EQUAL;
+	      if (a->flags & PORTCFG_FLAG_ANY || b->flags & PORTCFG_FLAG_ANY)
+		return PORTCFG_MATCH;
+	    }
 	  break;
 	case PROTO_ICMP:
-	  if (!strcmp (a->icmp_ipaddr, b->icmp_ipaddr) &&
-	      a->icmp_type == b->icmp_type)
-	    return 1;
+	  if (a->icmp_type == b->icmp_type)
+	    {
+	      if ((a->flags & PORTCFG_FLAG_DEVICE) || 
+		  (b->flags & PORTCFG_FLAG_DEVICE))
+		{
+		  if ((a->flags & PORTCFG_FLAG_DEVICE) && 
+		      (b->flags & PORTCFG_FLAG_DEVICE) &&
+		      !strcmp (svz_portcfg_device (a), svz_portcfg_device (b)))
+		    return PORTCFG_EQUAL;
+		  return PORTCFG_CONFLICT;
+		}
+	      if (a_addr->sin_addr.s_addr == b_addr->sin_addr.s_addr)
+		return PORTCFG_EQUAL;
+	      if (a->flags & PORTCFG_FLAG_ANY || b->flags & PORTCFG_FLAG_ANY)
+		return PORTCFG_MATCH;
+	    }
 	  break;
 	case PROTO_RAW:
-	  if (!strcmp (a->raw_ipaddr, b->raw_ipaddr))
-	    return 1;
+	  if ((a->flags & PORTCFG_FLAG_DEVICE) || 
+	      (b->flags & PORTCFG_FLAG_DEVICE))
+	    {
+	      if ((a->flags & PORTCFG_FLAG_DEVICE) && 
+		  (b->flags & PORTCFG_FLAG_DEVICE) &&
+		  !strcmp (svz_portcfg_device (a), svz_portcfg_device (b)))
+		return PORTCFG_EQUAL;
+	      return PORTCFG_CONFLICT;
+	    }
+	  if (a_addr->sin_addr.s_addr == b_addr->sin_addr.s_addr)
+	    return PORTCFG_EQUAL;
+	  if (a->flags & PORTCFG_FLAG_ANY || b->flags & PORTCFG_FLAG_ANY)
+	    return PORTCFG_MATCH;
 	  break;
 	}
-    } 
+    }
   else if (a->proto & PROTO_PIPE && a->proto == b->proto) 
     {
       /* 
@@ -111,11 +154,11 @@ svz_portcfg_equal (svz_portcfg_t *a, svz_portcfg_t *b)
        */
       if (!strcmp (a->pipe_recv.name, b->pipe_recv.name) && 
 	  !strcmp (b->pipe_send.name, b->pipe_send.name))
-	return 1;
+	return PORTCFG_EQUAL;
     } 
 
   /* Do not even the same proto flag -> cannot be equal. */
-  return 0;
+  return PORTCFG_NOMATCH;
 }
 
 /*
@@ -136,7 +179,8 @@ svz_portcfg_add (char *name, svz_portcfg_t *port)
   /* Check if the port configuration hash is inited. */
   if (svz_portcfgs == NULL)
     {
-      if ((svz_portcfgs = svz_hash_create (4)) == NULL)
+      if ((svz_portcfgs = svz_hash_create (4, (svz_free_func_t) 
+					   svz_portcfg_free)) == NULL)
 	return NULL;
     }
 
@@ -220,7 +264,7 @@ svz_portcfg_expand (svz_portcfg_t *this)
 
   /* Is this a network port configuration and should it be expanded ? */
   if ((addr = svz_portcfg_addr (this)) != NULL && 
-      addr->sin_addr.s_addr == INADDR_ANY)
+      (this->flags & PORTCFG_FLAG_ALL) && !(this->flags & PORTCFG_FLAG_DEVICE))
     {
       svz_interface_foreach (ifc, n)
 	{
@@ -264,15 +308,19 @@ svz_portcfg_dup (svz_portcfg_t *port)
     {
     case PROTO_TCP:
       copy->tcp_ipaddr = svz_strdup (port->tcp_ipaddr);
+      copy->tcp_device = svz_strdup (port->tcp_device);
       break;
     case PROTO_UDP:
       copy->udp_ipaddr = svz_strdup (port->udp_ipaddr);
+      copy->udp_device = svz_strdup (port->udp_device);
       break;
     case PROTO_ICMP:
       copy->icmp_ipaddr = svz_strdup (port->icmp_ipaddr);
+      copy->icmp_device = svz_strdup (port->icmp_device);
       break;
     case PROTO_RAW:
       copy->raw_ipaddr = svz_strdup (port->raw_ipaddr);
+      copy->raw_device = svz_strdup (port->raw_device);
       break;
     case PROTO_PIPE:
       copy->pipe_recv.name = svz_strdup (port->pipe_recv.name);
@@ -293,6 +341,53 @@ svz_portcfg_dup (svz_portcfg_t *port)
   return copy;
 }
 
+/*
+ * This function frees all resources allocated by the given port 
+ * configuration @var{port}.
+ */
+void
+svz_portcfg_free (svz_portcfg_t *port)
+{
+  /* Free the name of the port configuration. */
+  svz_free (port->name);
+
+  /* Depending on the type of configuration perform various operations. */
+  switch (port->proto)
+    {
+    case PROTO_TCP:
+      svz_free (port->tcp_ipaddr);
+      svz_free (port->tcp_device);
+      break;
+    case PROTO_UDP:
+      svz_free (port->udp_ipaddr);
+      svz_free (port->udp_device);
+      break;
+    case PROTO_ICMP:
+      svz_free (port->icmp_ipaddr);
+      svz_free (port->icmp_device);
+      break;
+    case PROTO_RAW:
+      svz_free (port->raw_ipaddr);
+      svz_free (port->raw_device);
+      break;
+    case PROTO_PIPE:
+      svz_free (port->pipe_recv.user);
+      svz_free (port->pipe_recv.name);
+      svz_free (port->pipe_recv.group);
+      svz_free (port->pipe_send.user);
+      svz_free (port->pipe_send.name);
+      svz_free (port->pipe_send.group);
+      break;
+    }
+
+  /* Destroy access and connection list. */
+  svz_portcfg_destroy_access (port);
+  svz_portcfg_destroy_accepted (port);
+
+  /* Free the port configuration itself. */
+  svz_free (port);
+}
+
 /*
  * This function makes the given port configuration @var{port} completely 
  * unusable. No operation is performed if @var{port} is @code{NULL}. If the
@@ -312,40 +407,8 @@ svz_portcfg_destroy (svz_portcfg_t *port)
   if (svz_portcfgs && (name = svz_hash_contains (svz_portcfgs, port)) != NULL)
     svz_hash_delete (svz_portcfgs, name);
 
-  /* Free the name of the port configuration. */
-  svz_free (port->name);
-
-  /* Depending on the type of configuration perform various operations. */
-  switch (port->proto)
-    {
-    case PROTO_TCP:
-      svz_free (port->tcp_ipaddr);
-      break;
-    case PROTO_UDP:
-      svz_free (port->udp_ipaddr);
-      break;
-    case PROTO_ICMP:
-      svz_free (port->icmp_ipaddr);
-      break;
-    case PROTO_RAW:
-      svz_free (port->raw_ipaddr);
-      break;
-    case PROTO_PIPE:
-      svz_free (port->pipe_recv.user);
-      svz_free (port->pipe_recv.name);
-      svz_free (port->pipe_recv.group);
-      svz_free (port->pipe_send.user);
-      svz_free (port->pipe_send.name);
-      svz_free (port->pipe_send.group);
-      break;
-    }
-
-  /* Destroy access and connection list. */
-  svz_portcfg_destroy_access (port);
-  svz_portcfg_destroy_accepted (port);
-
-  /* Free the port configuration itself. */
-  svz_free (port);
+  /* Free the port configuration. */
+  svz_portcfg_free (port);
 }
 
 /*
@@ -374,13 +437,8 @@ svz_portcfg_destroy_access (svz_portcfg_t *port)
 void
 svz_portcfg_destroy_accepted (svz_portcfg_t *port)
 {
-  int n;
-  char **ip;
-
   if (port->accepted)
     {
-      svz_hash_foreach_key (port->accepted, ip, n)
-	svz_vector_destroy (svz_hash_get (port->accepted, ip[n]));
       svz_hash_destroy (port->accepted);
       port->accepted = NULL;
     }
@@ -407,18 +465,8 @@ svz_portcfg_get (char *name)
 void
 svz_portcfg_finalize (void)
 {
-  svz_portcfg_t **port;
-  int n, i;
-
   if (svz_portcfgs != NULL)
     {
-      n = svz_hash_size (svz_portcfgs) - 1;
-      svz_hash_foreach_value (svz_portcfgs, port, i)
-	{
-	  svz_portcfg_destroy (port[n]);
-	  n--;
-	  i--;
-	}
       svz_hash_destroy (svz_portcfgs);
       svz_portcfgs = NULL;
     }
@@ -436,6 +484,10 @@ svz_portcfg_convert_addr (char *str, struct sockaddr_in *addr)
 
   if ((ifc = svz_interface_search (str)) != NULL)
     {
+#if ENABLE_DEBUG
+      svz_log (LOG_DEBUG, "`%s' is %s\n", ifc->description, 
+	       svz_inet_ntoa (ifc->ipaddr));
+#endif
       addr->sin_addr.s_addr = ifc->ipaddr;
       return 0;
     }
@@ -457,16 +509,32 @@ svz_portcfg_mkaddr (svz_portcfg_t *this)
       /* For all network protocols we assign AF_INET as protocol family,
 	 determine the network port (if necessary) and put the ip address. */
     case PROTO_TCP:
-      err = svz_portcfg_convert_addr (this->tcp_ipaddr, &this->tcp_addr);
       this->tcp_addr.sin_family = AF_INET;
-      if (!(this->tcp_port > 0 && this->tcp_port < 65536))
+      if (svz_portcfg_device (this))
 	{
-	  svz_log (LOG_ERROR, "%s: TCP port requires a short (1..65535)\n",
-		   this->name);
-	  err = -1;
+	  this->flags |= PORTCFG_FLAG_DEVICE;
+	  this->tcp_addr.sin_addr.s_addr = INADDR_ANY;
+	}
+      else if (!strcmp (this->tcp_ipaddr, PORTCFG_ANY))
+	{
+	  this->flags |= PORTCFG_FLAG_ANY;
+	  this->tcp_addr.sin_addr.s_addr = INADDR_ANY;
+	}
+      else if (!strcmp (this->tcp_ipaddr, PORTCFG_NOIP))
+	{
+	  this->flags |= PORTCFG_FLAG_ALL;
+	  this->tcp_addr.sin_addr.s_addr = INADDR_ANY;
 	}
       else
-	this->tcp_addr.sin_port = htons (this->tcp_port);
+	{
+	  err = svz_portcfg_convert_addr (this->tcp_ipaddr, &this->tcp_addr);
+	  if (err)
+	    {
+	      svz_log (LOG_ERROR, "%s: `%s' is not a valid IP address\n",
+		       this->name, this->tcp_ipaddr);
+	    }
+	}
+      this->tcp_addr.sin_port = htons (this->tcp_port);
       if (this->tcp_backlog > SOMAXCONN)
 	{
 	  svz_log (LOG_ERROR, "%s: TCP backlog out of range (1..%d)\n",
@@ -475,23 +543,65 @@ svz_portcfg_mkaddr (svz_portcfg_t *this)
 	}
       break;
     case PROTO_UDP:
-      err = svz_portcfg_convert_addr (this->udp_ipaddr, &this->udp_addr);
       this->udp_addr.sin_family = AF_INET;
-      if (!(this->udp_port > 0 && this->udp_port < 65536))
+      if (svz_portcfg_device (this))
 	{
-	  svz_log (LOG_ERROR, "%s: UDP port requires a short (1..65535)\n",
-		   this->name);
-	  err = -1;
+	  this->flags |= PORTCFG_FLAG_DEVICE;
+	  this->udp_addr.sin_addr.s_addr = INADDR_ANY;
+	}
+      else if (!strcmp (this->udp_ipaddr, PORTCFG_ANY))
+	{
+	  this->flags |= PORTCFG_FLAG_ANY;
+	  this->udp_addr.sin_addr.s_addr = INADDR_ANY;
+	}
+      else if (!strcmp (this->tcp_ipaddr, PORTCFG_NOIP))
+	{
+	  this->flags |= PORTCFG_FLAG_ALL;
+	  this->udp_addr.sin_addr.s_addr = INADDR_ANY;
 	}
       else
-	this->udp_addr.sin_port = htons (this->udp_port);
+	{
+	  err = svz_portcfg_convert_addr (this->udp_ipaddr, &this->udp_addr);
+	  if (err)
+	    {
+	      svz_log (LOG_ERROR, "%s: `%s' is not a valid IP address\n",
+		       this->name, this->udp_ipaddr);
+	    }
+	}
+      this->udp_addr.sin_port = htons (this->udp_port);
       break;
     case PROTO_ICMP:
-      err = svz_portcfg_convert_addr (this->icmp_ipaddr, &this->icmp_addr);
+      if (svz_portcfg_device (this))
+	{
+	  this->flags |= PORTCFG_FLAG_DEVICE;
+	  this->icmp_addr.sin_addr.s_addr = INADDR_ANY;
+	}
+      else
+	{
+	  err = svz_portcfg_convert_addr (this->icmp_ipaddr, &this->icmp_addr);
+	  if (err)
+	    {
+	      svz_log (LOG_ERROR, "%s: `%s' is not a valid IP address\n",
+		       this->name, this->icmp_ipaddr);
+	    }
+	}
       this->icmp_addr.sin_family = AF_INET;
       break;
     case PROTO_RAW:
-      err = svz_portcfg_convert_addr (this->raw_ipaddr, &this->raw_addr);
+      if (svz_portcfg_device (this))
+	{
+	  this->flags |= PORTCFG_FLAG_DEVICE;
+	  this->raw_addr.sin_addr.s_addr = INADDR_ANY;
+	}
+      else
+	{      
+	  err = svz_portcfg_convert_addr (this->raw_ipaddr, &this->raw_addr);
+	  if (err)
+	    {
+	      svz_log (LOG_ERROR, "%s: `%s' is not a valid IP address\n",
+		       this->name, this->raw_ipaddr);
+	    }
+	}
       this->raw_addr.sin_family = AF_INET;
       break;
       /* The pipe protocol needs a check for the validity of the permissions,
@@ -532,8 +642,8 @@ svz_portcfg_prepare (svz_portcfg_t *port)
 	port->detection_wait = SOCK_MAX_DETECTION_WAIT;
     }
   /* Check the initial send and receive buffer sizes. */
-  if (port->send_buffer_size <= 0
-      /* FIXME: || port->send_buffer_size >= REAL_BIG_VALUE */)
+  if (port->send_buffer_size <= 0 || 
+      port->send_buffer_size >= MAX_BUF_SIZE)
     {
       if (port->proto & (PROTO_TCP | PROTO_PIPE))
 	port->send_buffer_size = SEND_BUF_SIZE;
@@ -542,8 +652,8 @@ svz_portcfg_prepare (svz_portcfg_t *port)
       else if (port->proto & (PROTO_ICMP | PROTO_RAW))
 	port->send_buffer_size = ICMP_BUF_SIZE;
     }
-  if (port->recv_buffer_size <= 0
-      /* FIXME: || port->recv_buffer_size >= REAL_BIG_VALUE */)
+  if (port->recv_buffer_size <= 0 ||
+      port->recv_buffer_size >= MAX_BUF_SIZE)
     {
       if (port->proto & (PROTO_TCP | PROTO_PIPE))
 	port->recv_buffer_size = RECV_BUF_SIZE;
@@ -555,11 +665,74 @@ svz_portcfg_prepare (svz_portcfg_t *port)
   /* Check the connection frequency. */
   if (port->connect_freq <= 0)
     {
-      /* FIXME: sane value is ? */
+      /* Sane value is: 100 connections per second. */
       port->connect_freq = 100;
     }
 }
 
+/*
+ * Helper function for the below routine. Converts a Internet network 
+ * address or the appropiate device into a text representation.
+ */
+static char *
+svz_portcfg_addr_text (svz_portcfg_t *port, struct sockaddr_in *addr)
+{
+  if (port->flags & PORTCFG_FLAG_DEVICE)
+    return svz_portcfg_device (port);
+  else if (port->flags & PORTCFG_FLAG_ANY)
+    return "*";
+  return svz_inet_ntoa (addr->sin_addr.s_addr);
+}
+
+/*
+ * This function returns a simple text representation of the given port
+ * configuration @var{port}. The returned character string is statically
+ * allocated. Thus you cannot use it twice in argument lists.
+ */
+char *
+svz_portcfg_text (svz_portcfg_t *port)
+{
+  static char text[128];
+  struct sockaddr_in *addr;
+
+  /* Wipe the text. */
+  text[0] = '\0';
+
+  /* TCP and UDP */ 
+  if (port->proto & (PROTO_TCP | PROTO_UDP))
+    {
+      addr = svz_portcfg_addr (port);
+      strcat (text, (port->proto & PROTO_TCP) ? "TCP:[" : "UDP:[");
+      strcat (text, svz_portcfg_addr_text (port, addr));
+      strcat (text, ":");
+      strcat (text, svz_itoa (ntohs (addr->sin_port)));
+      strcat (text, "]");
+    }
+  /* RAW and ICMP */
+  else if (port->proto & (PROTO_RAW | PROTO_ICMP))
+    {
+      addr = svz_portcfg_addr (port);
+      strcat (text, (port->proto & PROTO_RAW) ? "RAW:[" : "ICMP:[");
+      strcat (text, svz_portcfg_addr_text (port, addr));
+      if (port->proto & PROTO_ICMP)
+	{
+	  strcat (text, "/");
+	  strcat (text, svz_itoa (port->icmp_type));
+	}
+      strcat (text, "]");
+    }
+  /* PIPE */
+  else if (port->proto & PROTO_PIPE)
+    {
+      strcat (text, "PIPE:[");
+      strcat (text, port->pipe_recv.name);
+      strcat (text, "]-[");
+      strcat (text, port->pipe_send.name);
+      strcat (text, "]");
+    }
+  return text;
+}
+
 /*
  * Debug helper: Emit a printable representation of the the given
  * port configuration to the given FILE stream @var{f}.
