@@ -1,7 +1,7 @@
 /*
  * server-core.c - server core implementation
  *
- * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  * Copyright (C) 2000 Raimund Jacob <raimi@lkcc.org>
  * Copyright (C) 1999 Martin Grabmueller <mgrabmue@cs.tu-berlin.de>
  *
@@ -20,7 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: server-core.c,v 1.32 2001/11/25 03:38:23 ela Exp $
+ * $Id: server-core.c,v 1.37 2002/02/15 12:16:07 ela Exp $
  *
  */
 
@@ -34,7 +34,9 @@
 
 /* Some Unices define the strsignal() function depending on 
    this definition. */
-#define __EXTENSIONS__
+#ifndef __EXTENSIONS__
+# define __EXTENSIONS__
+#endif
 #include <string.h>
 
 #include <errno.h>
@@ -87,6 +89,7 @@
 #include "libserveez/pipe-socket.h"
 #include "libserveez/server-loop.h"
 #include "libserveez/portcfg.h"
+#include "libserveez/interface.h"
 #include "libserveez/coserver/coserver.h"
 #include "libserveez/server.h"
 #include "libserveez/server-core.h"
@@ -624,7 +627,7 @@ svz_sock_check_frequency (svz_socket_t *parent, svz_socket_t *child)
   if (port->accepted)
     accepted = (svz_vector_t *) svz_hash_get (port->accepted, ip);
   else
-    port->accepted = svz_hash_create (4);
+    port->accepted = svz_hash_create (4, (svz_free_func_t) svz_vector_destroy);
   current = time (NULL);
 
   if (accepted != NULL)
@@ -803,7 +806,7 @@ svz_sock_find (int id, int version)
 
   if (id & ~(svz_sock_limit - 1))
     {
-      svz_log (LOG_FATAL, "socket id %d is invalid\n", id);
+      svz_log (LOG_WARNING, "socket id %d is invalid\n", id);
       return NULL;
     }
 
@@ -885,6 +888,7 @@ static int
 svz_reset (void)
 {
   /* FIXME: Maybe `server_t' reset callback ? */
+  svz_interface_check ();
   return 0;
 }
 
@@ -894,7 +898,7 @@ svz_reset (void)
  * and all memory used by the socket gets freed. Note that this
  * function calls the @var{sock}'s disconnect handler if defined.
  */
-static int
+int
 svz_sock_shutdown (svz_socket_t *sock)
 {
 #if ENABLE_DEBUG
