@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: server.c,v 1.31 2001/11/23 13:18:39 ela Exp $
+ * $Id: server.c,v 1.33 2001/12/15 02:47:38 ela Exp $
  *
  */
 
@@ -326,7 +326,7 @@ svz_server_t *
 svz_server_add (svz_server_t *server)
 {
   if (svz_servers == NULL)
-    svz_servers = svz_hash_create (4);
+    svz_servers = svz_hash_create (4, (svz_free_func_t) svz_server_finalize);
   return svz_hash_put (svz_servers, server->name, server);
 }
 
@@ -603,7 +603,7 @@ svz_hash_t *
 svz_config_hash_create (char **strarray)
 {
   int i;
-  svz_hash_t *hash = svz_hash_create (4);
+  svz_hash_t *hash = svz_hash_create (4, svz_free);
 
   if (strarray)
     {
@@ -626,15 +626,7 @@ svz_config_hash_create (char **strarray)
 void
 svz_config_hash_destroy (svz_hash_t *strhash)
 {
-  char **strarray;
-  int i;
-
-  if (strhash)
-    {
-      svz_hash_foreach_value (strhash, strarray, i)
-	svz_free (strarray[i]);
-      svz_hash_destroy (strhash);
-    }
+  svz_hash_destroy (strhash);
 }
 
 /*
@@ -651,7 +643,7 @@ svz_config_hash_dup (svz_hash_t *strhash)
 
   if (strhash)
     {
-      hash = svz_hash_create (4);
+      hash = svz_hash_create (4, strhash->destroy);
       svz_hash_foreach_key (strhash, keys, i)
 	{
 	  svz_hash_put (hash, keys[i], 
@@ -888,6 +880,24 @@ svz_server_configure (svz_servertype_t *server,
   return cfg;
 }
 
+/*
+ * This function runs the server initilizer of the given server instance
+ * @var{server} and returns zero on success. Otherwise it emits an error
+ * message and returns non-zero.
+ */
+int
+svz_server_init (svz_server_t *server)
+{
+  if (server)
+    if (server->init != NULL) 
+      if (server->init (server) < 0)
+	{
+	  svz_log (LOG_ERROR, "error initializing `%s'\n", server->name);
+	  return -1;
+	}
+  return 0;
+}
+
 /*
  * Run the initializers of all servers, return -1 if some server did not
  * think it is a good idea to run.
@@ -900,39 +910,37 @@ svz_server_init_all (void)
 
   svz_log (LOG_NOTICE, "initializing all server instances\n");
   svz_hash_foreach_value (svz_servers, server, i)
-    {
-      if (server[i]->init != NULL) 
-	if (server[i]->init (server[i]) < 0) 
-	  {
-	    errneous = -1;
-	    svz_log (LOG_ERROR, "error initializing `%s'\n", server[i]->name);
-	  }
-    }
+    if (svz_server_init (server[i]) < 0)
+      errneous = -1;
   return errneous;
 }
 
+/*
+ * This function runs the finalizer callback for the given server instance
+ * @var{server}, removes all bindings and frees all resources allocated by
+ * the server instance.
+ */
+void
+svz_server_finalize (svz_server_t *server)
+{
+  if (server)
+    {
+      if (server->finalize != NULL)
+	if (server->finalize (server) < 0)
+	  svz_log (LOG_ERROR, "error finalizing `%s'\n", server->name);
+      svz_server_unbind (server);
+      svz_server_free (server);
+    }
+}
+
 /*
  * Run the local finalizers for all server instances.
  */
 int
 svz_server_finalize_all (void)
 {
-  int i, n;
-  svz_server_t **server;
-
   svz_log (LOG_NOTICE, "running all server finalizers\n");
-  n = svz_hash_size (svz_servers) - 1;
-  svz_hash_foreach_value (svz_servers, server, i)
-    {
-      if (server[n]->finalize != NULL)
-	if (server[n]->finalize (server[n]) < 0)
-	  svz_log (LOG_ERROR, "error finalizing `%s'\n", server[n]->name);
-      svz_server_del (server[n]->name);
-      i--;
-      n--;
-    }
   svz_hash_destroy (svz_servers);
   svz_servers = NULL;
-
   return 0;
 }
