@@ -1,7 +1,7 @@
 /*
  * tunnel.c - port forward implementations
  *
- * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: tunnel.c,v 1.26 2001/07/03 10:43:31 ela Exp $
+ * $Id: tunnel.c,v 1.29 2002/01/22 20:27:00 ela Exp $
  *
  */
 
@@ -126,7 +126,7 @@ tnl_init (svz_server_t *server)
     }
 
   /* check identity of source and target port configurations */
-  if (svz_portcfg_equal (cfg->source, cfg->target))
+  if (svz_portcfg_equal (cfg->source, cfg->target) == PORTCFG_EQUAL)
     {
       svz_log (LOG_ERROR, "tunnel: source and target identical\n");
       return -1;
@@ -144,7 +144,7 @@ tnl_init (svz_server_t *server)
     }
 
   /* create source client hash (for UDP and ICMP only) */
-  cfg->client = svz_hash_create (4);
+  cfg->client = svz_hash_create (4, svz_free);
   
   /* assign the appropriate handle request routine of the server */
   if (cfg->source->proto & PROTO_UDP)
@@ -162,18 +162,8 @@ int
 tnl_finalize (svz_server_t *server)
 {
   tnl_config_t *cfg = server->cfg;
-  tnl_connect_t **source;
-  int n;
 
   /* release source connection hash if necessary */
-  if ((source = (tnl_connect_t **) svz_hash_values (cfg->client)) != NULL)
-    {
-      for (n = 0; n < svz_hash_size (cfg->client); n++)
-	{
-	  svz_free (source[n]);
-	}
-      svz_hash_xfree (source);
-    }
   svz_hash_destroy (cfg->client);
 
   return 0;
